// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>
#include <OB/Properties.h>

#include <OB/EventServerAdmin.h>

#include <stdlib.h>

#include <Config.h> // Make sure that the compiler complains if the
                    // configuration doesn't match

static void
usage(const char* progName)
{
    cerr << "Usage:\n";
    cerr << progName << " [options] shutdown\n"
	"\n"
	"Options:\n"
	"-h, --help             Show this message.\n"
	"-v, --version          Show version number.\n"
        "-t, --typed-service    Admin for typed event service.\n"
        "-u, --untyped-service  Adin for untyped event service [default].\n"
	;
}

int
main(int argc, char** argv)
{
    try
    {
	int i;

	//
	// Create ORB
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);


        //
	// Get options
	//
	for(i = 1 ; i < argc && *argv[i] == '-' ; i++)
	{
	    if(strcmp(argv[i], "--help") == 0 ||
	       strcmp(argv[i], "-h") == 0)
	    {
		usage(argv[0]);
		return 0;
	    }
	    else if(strcmp(argv[i], "--version") == 0 ||
		    strcmp(argv[i], "-v") == 0)
	    {
		cerr << "Event Service" << OBVersion << endl;
		return 0;
	    }
	    else
	    {
		cerr << argv[0] << ": unknown option `"
		     << argv[i] << "'\n" << endl;
		usage(argv[0]);
		return 1;
	    }
	}
        
        CORBA_Object_var obj;
        try
	{
            obj =
                orb -> resolve_initial_references("EventServerAdmin");
        }
        catch(CORBA_ORB::InvalidName&)
        {
        }

        if(CORBA_is_nil(obj))
        {
            cerr << "No `EventServerAdmin' initial reference." << endl;
            exit(1);
        }
        EventServerAdmin_var admin = EventServerAdmin::_narrow(obj);
        if(CORBA_is_nil(admin))
        {
            cerr << "`EventServerAdmin' initial reference in invalid."
                 << endl;
            exit(1);
            
        }
        
        if(i < argc && strcmp(argv[i], "shutdown") == 0)
        {
            admin -> shutdown();
        }
        else
        {
            usage(argv[0]);
        }
    }
    catch(const CORBA_SystemException& ex)
    {
        OBPrintException(ex);
        return 1;
    }
    return 0;
}
