// **********************************************************************
//
// Copyright (c) 1999
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1999
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#ifndef EXT_NAMING_IMPL_H
#define EXT_NAMING_IMPL_H

//
// Forward declarations
//
class NamingDatabase;
class NamingContextSet;
class NcOrObj;
struct NcOrObjBinding;

class CosNaming_ExtNamingContext_impl
    : virtual public CosNaming_ExtNamingContext_skel
      OB_COMMA_MONITOR
{
    static int count_; // Running static count for keys

    CORBA_ORB_var orb_;
    CORBA_String_var key_;

public: // Some compilers need this to be public

    typedef OBHashtable<CosNaming_NameComponent, NcOrObjBinding*> BindingsHash;

private:

    //
    // Set of bindings in this context.
    //
    BindingsHash* bindings_;
    NamingContextSet& ncs_;
    NamingDatabase* store_;
    bool destroyed_; // Has the context been destroyed?

public: // Some compilers need this to be public

    typedef OBHashtable<CORBA_String_var, CORBA_Object_var > ObjectHash;

private:
    
    //
    // Clients to which this service reports events
    //
    static CosNaming_BindingListenerSeq listeners_;

    void doBind(const CosNaming_Name& n, const NcOrObj& ncOrObj, bool, int);

    //
    // Support for BindingListener interface
    //
    void bindingAdded(const CosNaming_Name& n);
    void bindingRemoved(const CosNaming_Name& n, bool isContext);
    void bindingChanged(const CosNaming_Name& n, bool isContext);

public:

    CosNaming_ExtNamingContext_impl(CORBA_ORB_ptr,
                                 NamingDatabase*,
                                 NamingContextSet&,
                                 const char*);
    ~CosNaming_ExtNamingContext_impl();
    
    virtual void bind(const CosNaming_Name& n,
                      CORBA_Object_ptr obj);
    virtual void bind(const CosNaming_Name& n,
                      CORBA_Object_ptr obj, int timestamp);

    virtual void rebind(const CosNaming_Name& n,
                        CORBA_Object_ptr obj);

    virtual void bind_context(const CosNaming_Name& n,
                              CosNaming_NamingContext_ptr nc);
    virtual void bind_context(const CosNaming_Name& n,
                              CosNaming_NamingContext_ptr nc,
			      int timestamp);

    virtual void rebind_context(const CosNaming_Name& n,
                                CosNaming_NamingContext_ptr nc);

    virtual CORBA_Object_ptr resolve(const CosNaming_Name& n);

    virtual void unbind(const CosNaming_Name& n);

    virtual CosNaming_NamingContext_ptr new_context();

    virtual CosNaming_NamingContext_ptr bind_new_context(const
							 CosNaming_Name& n);

    virtual void destroy();

    virtual void list(CORBA_ULong how_many,
                      CosNaming_BindingList*& bl,
                      CosNaming_BindingIterator_ptr& bi);

    const char* getKey() const { return key_; }
    OB_MAKE_NARROW_IMPL(CosNaming_ExtNamingContext_impl)

    virtual void rename(const CosNaming_Name& old_name,
                        const CosNaming_Name& new_name);

    virtual void add_client(CosNaming_BindingListener_ptr client);

    virtual void remove_client(CosNaming_BindingListener_ptr client);

    virtual CosNaming_ExtNamingContext::ExtendedBindingList* list_all();
};

class CosNaming_BindingIterator_impl
    : virtual public CosNaming_BindingIterator_skel
      OB_COMMA_MONITOR
{
    CORBA_ORB_var orb_;
    CORBA_ULong idx_;
    CosNaming_BindingList bl_;

public:

    CosNaming_BindingIterator_impl(CORBA_ORB_ptr orb,
				   const CosNaming_BindingList&);

    virtual CORBA_Boolean next_one(CosNaming_Binding*& b);
    virtual CORBA_Boolean next_n(CORBA_ULong how_many,
                                 CosNaming_BindingList*& bl);

    virtual void destroy();
};

#endif
