// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>
#include <OB/Timer.h>
#include <OB/CosNaming.h>

#include <stdio.h>
#include <NamingDatabase.h>

#include <Logger.h>

#ifdef HAVE_JTC

Logger::Logger(NamingDatabase* store, long interval)
    : store_(store), interval_(interval), stop_(false)
{
}

void
Logger::halt()
{
    JTCSynchronized sync(*this);
    stop_ = true;
    notify();
}

void
Logger::run()
{
    JTCSynchronized sync(*this);
    while(!stop_)
    {
	try
	{
	    wait(interval_*1000);
	}
	catch(const JTCInterruptedException&)
	{
	}
	if(stop_)
	    continue;
#ifdef OB_TRACE
	OBMessageViewer::instance() -> trace(2, "Compacting database");
#endif
	store_ -> updateJournal();
    }
}

#else

Logger::Logger(NamingDatabase* store, long logSeconds)
    : store_(store)
{
    interval_.tv_sec = logSeconds;
    interval_.tv_usec = 0;
    activate(interval_, true);
}

void
Logger::notify()
{
#ifdef OB_TRACE
    OBMessageViewer::instance() -> trace(2, "Compacting database");
#endif
    store_ -> updateJournal();
    activate(interval_, true);
}

#endif
