// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <string.h>
#include "TestListener_impl.h"

TestListener_impl::TestListener_impl() :
    op_(TestListener_impl::OP_NONE), ctx_(CosNaming_NamingContext::_nil()),
    valid_(false)
{
}

void
TestListener_impl::expect(TestListener_impl::Operation op,
                          CosNaming_NamingContext_ptr ctx,
                          const CosNaming_NameComponent& name)

{
    op_ = op;
    ctx_ = CosNaming_NamingContext::_duplicate(ctx);
    name_ = name;
}

bool
TestListener_impl::isValid()
{
    if(valid_)
    {
	reset();
	return true;
    }
    else
	return false;
}

//
// CosNaming::BindingListener operations
//

void
TestListener_impl::context_added(CosNaming_NamingContext_ptr ctx,
                                 const CosNaming_NameComponent& name)
{
    validate(CONTEXT_ADDED, ctx, name);
}

void
TestListener_impl::context_removed(CosNaming_NamingContext_ptr ctx,
                                   const CosNaming_NameComponent& name)
{
    validate(CONTEXT_REMOVED, ctx, name);
}

void
TestListener_impl::context_changed(CosNaming_NamingContext_ptr ctx,
                                   const CosNaming_NameComponent& name)
{
    validate(CONTEXT_CHANGED, ctx, name);
}

void
TestListener_impl::object_added(CosNaming_NamingContext_ptr ctx,
                                const CosNaming_NameComponent& name)
{
    validate(OBJECT_ADDED, ctx, name);
}

void
TestListener_impl::object_removed(CosNaming_NamingContext_ptr ctx,
                                  const CosNaming_NameComponent& name)
{
    validate(OBJECT_REMOVED, ctx, name);
}

void
TestListener_impl::object_changed(CosNaming_NamingContext_ptr ctx,
                                  const CosNaming_NameComponent& name)
{
    validate(OBJECT_CHANGED, ctx, name);
}

//
// Private functions
//

void
TestListener_impl::validate(TestListener_impl::Operation op,
                            CosNaming_NamingContext_ptr ctx,
                            const CosNaming_NameComponent& name)
{
    assert(op_ == op);
    assert(ctx_ -> _is_equivalent(ctx));
    assert(equal(name_, name));
    valid_ = true;
}

bool
TestListener_impl::equal(const CosNaming_NameComponent& n1,
                         const CosNaming_NameComponent& n2)
{
    return(strcmp(n1.id, n2.id) == 0 && strcmp(n1.kind, n2.kind) == 0);
}

void
TestListener_impl::reset()
{
    op_ = OP_NONE;
    valid_ = false;
}
