// **********************************************************************
//
// Copyright (c) 1998
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// Copyright (c) 1998
// Object-Oriented Concepts GmbH
// Ettlingen, Germany
//
// All Rights Reserved
//
// **********************************************************************

#include <assert.h>
#include <stdlib.h>
#include <string.h>

#include <OB/CORBA.h>
#include <OB/Util.h>

#include <OB/CosNaming.h>

#include "TestObject_impl.h"
#include "TestMacro.h"


//
// Test CosNaming::NamingContext::rebind()
//
void
TestRebind(CORBA_ORB_ptr orb, CosNaming_NamingContext_ptr root)
{
    TestObject_var test = new TestObject_impl;
    CosNaming_Name name1, name2;

    //
    // Test exceptions
    //

    try
    {
        //
        // Try to rebind an object when an intermediate context
        // doesn't exist
        //
        name1.length(2);
        name1[0].id = CORBA_string_dup("bad");
        name1[0].kind = CORBA_string_dup("node");
        name1[1].id = CORBA_string_dup("my");
        name1[1].kind = CORBA_string_dup("node");
        root -> rebind(name1, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::NotFound& ex)
    {
        assert(ex.why == CosNaming_NamingContext::missing_node);
    }

    name1.length(1);
    name1[0].id = CORBA_string_dup("object");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(root -> bind(name1, test));

    try
    {
        //
        // Try to rebind an object when an object is in the path
        //
        name2.length(2);
        name2[0].id = CORBA_string_dup("object");
        name2[0].kind = CORBA_string_dup("");
        name2[1].id = CORBA_string_dup("my");
        name2[1].kind = CORBA_string_dup("node");
        root -> rebind(name2, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::NotFound& ex)
    {
        assert(ex.why == CosNaming_NamingContext::not_context);
    }

    //
    // Rebind nil object to name
    //
    SUCCEED(root -> rebind(name1, CORBA_Object::_nil()));
    SUCCEED(root -> unbind(name1));

    try
    {
        //
        // Try to rebind an object with an invalid name
        //
        name1.length(0);
        root -> rebind(name1, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::InvalidName&)
    {
        // expected
    }

    try
    {
        //
        // Try to rebind an object with an invalid name
        //
        name1.length(1);
        name1[0].id = CORBA_string_dup("");
        name1[0].kind = CORBA_string_dup("");
        root -> rebind(name1, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::InvalidName&)
    {
        // expected
    }

    //
    // Bind an intermediate context for use in more tests
    //
    CosNaming_NamingContext_var context = root -> new_context();
    name1.length(1);
    name1[0].id = CORBA_string_dup("ctx");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(root -> bind_context(name1, context));

    try
    {
        //
        // Try to rebind an object when an intermediate context
        // doesn't exist
        //
        name1.length(3);
        name1[0].id = CORBA_string_dup("ctx");
        name1[0].kind = CORBA_string_dup("");
        name1[1].id = CORBA_string_dup("bad");
        name1[1].kind = CORBA_string_dup("node");
        name1[2].id = CORBA_string_dup("my");
        name1[2].kind = CORBA_string_dup("node");
        root -> rebind(name1, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::NotFound& ex)
    {
        assert(ex.why == CosNaming_NamingContext::missing_node);
    }

    name1.length(1);
    name1[0].id = CORBA_string_dup("object");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(context -> bind(name1, test));

    try
    {
        //
        // Try to rebind an object when an object is in the path
        //
        name2.length(3);
        name2[0].id = CORBA_string_dup("ctx");
        name2[0].kind = CORBA_string_dup("");
        name2[1].id = CORBA_string_dup("object");
        name2[1].kind = CORBA_string_dup("");
        name2[2].id = CORBA_string_dup("my");
        name2[2].kind = CORBA_string_dup("node");
        root -> rebind(name2, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::NotFound& ex)
    {
        assert(ex.why == CosNaming_NamingContext::not_context);
    }

    //
    // Rebind nil object to an existing name
    //
    name2.length(2);
    name2[0].id = CORBA_string_dup("ctx");
    name2[0].kind = CORBA_string_dup("");
    name2[1].id = CORBA_string_dup("object");
    name2[1].kind = CORBA_string_dup("");
    SUCCEED(root -> rebind(name2, CORBA_Object::_nil()));

    SUCCEED(context -> unbind(name1));

    //
    // Rebind nil object to a name currently bound to a context
    //
    name1.length(1);
    name1[0].id = CORBA_string_dup("ctx");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(root -> rebind(name1, CORBA_Object::_nil()));

/* XXX Does the spec allow a context binding to be rebound to an object?
   XXX If it doesn't say, should we allow it?
    try
    {
        //
        // Try to rebind an object using the name of an existing context
        //
        name1.length(1);
        name1[0].id = CORBA_string_dup("ctx");
        name1[0].kind = CORBA_string_dup("");
        root -> rebind(name1, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::NotFound& ex)
    {
        assert(ex.why == CosNaming_NamingContext::not_object);
    }
*/

    //
    // Clean up context
    //
    name1.length(1);
    name1[0].id = CORBA_string_dup("ctx");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(root -> unbind(name1));
    SUCCEED(context -> destroy());


    //
    // Test name equality to make sure kind is included in comparisons
    //
    name1.length(1);
    name1[0].id = CORBA_string_dup("object");
    name1[0].kind = CORBA_string_dup("kind1");
    name2.length(1);
    name2[0].id = CORBA_string_dup("object");
    name2[0].kind = CORBA_string_dup("kind2");
    SUCCEED(root -> rebind(name1, test));
    SUCCEED(root -> rebind(name2, test));
    SUCCEED(root -> unbind(name1));
    SUCCEED(root -> unbind(name2));


    //
    // Bind a nil context
    //
    name1.length(1);
    name1[0].id = CORBA_string_dup("nil_ctx");
    name1[0].kind = CORBA_string_dup("");
    SUCCEED(root -> bind_context(name1, CosNaming_NamingContext::_nil()));

    try
    {
        //
        // Try to rebind an object through a nil context
        //
        name2.length(2);
        name2[0].id = CORBA_string_dup("nil_ctx");
        name2[0].kind = CORBA_string_dup("");
        name2[1].id = CORBA_string_dup("object");
        name2[1].kind = CORBA_string_dup("");
        root -> rebind(name2, test);
        assert("rebind should not have succeeded" == 0);
    }
    catch(CosNaming_NamingContext::CannotProceed&)
    {
        // expected
    }

    SUCCEED(root -> unbind(name1));
}
