// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>
#include <Peer_impl.h>

#include <stdlib.h>
#include <errno.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

int
main(int argc, char* argv[], char*[])
{
    try
    {
	//
	// Create ORB and BOA
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);
	CORBA_BOA_var boa = orb -> BOA_init(argc, argv);
	
	//
	// Set the concurrency model
	//
#ifdef HAVE_JTC
	if(orb -> conc_model() != CORBA_ORB::ConcModelThreaded ||
	   (boa -> conc_model() != CORBA_BOA::ConcModelThreadPerRequest &&
	    boa -> conc_model() != CORBA_BOA::ConcModelThreadPool))
	{
	    orb -> conc_model(CORBA_ORB::ConcModelReactive);
	    boa -> conc_model(CORBA_BOA::ConcModelReactive);
	}
#else
	orb -> conc_model(CORBA_ORB::ConcModelReactive);
	boa -> conc_model(CORBA_BOA::ConcModelReactive);
#endif
	
	//
	// Create implementation object
	//
	Peer_var p1 = new Peer_impl;
    
	//
	// Save reference
	//
	CORBA_String_var s = orb -> object_to_string(p1);
	
	const char* refFile = "Peer.ref";
	ofstream out(refFile);
	if(out.fail())
	{
	    cerr << argv[0] << ": can't open `" << refFile << "': "
		 << strerror(errno) << endl;
	    return 1;
	}
	
	out << s << endl;
	out.close();
	
	//
	// Run implementation
	//
	boa -> impl_is_ready(CORBA_ImplementationDef::_nil());
    }
    catch(CORBA_SystemException& ex)
    {
	OBPrintException(ex);
	return 1;
    }

    return 0;
}
